<?php
/*
 * Mboard Helper Plugin.
 *
 * @license  Apache 2 (http://www.apache.org/licenses/LICENSE-2.0)
 * @author   Kazuyuki Matsuda / kazm.atmark.shorindo.com
 */

if (!defined('DOKU_INC')) die();
if (!defined('DOKU_LF')) define('DOKU_LF', "\n");
if (!defined('DOKU_TAB')) define('DOKU_TAB', "\t");

class helper_plugin_mboard extends DokuWiki_Plugin {
    var $message;
    var $_conn;

    function __construct() {
        $datasource = plugin_load('helper', 'datasource');
        if (!is_object($datasource))
            throw new Exception('datasource plugin is not available!');
        $this->_conn = $datasource->connect();
        /*
        $this->_conn = new PDO($this->getConf('dsn'),
                       $this->getConf('dbuser'),
                       conf_decodeString($this->getConf('dbpass')));
        */
    }

    function getInfo(){
        return confToHash(dirname(__FILE__).'/plugin.info.txt');
    }

    function getFormat(){
        return 'mboard';
    }

    function canRender($format) {
        return ($format == 'mboard');
    }

    function getMethods() {
        return array();
    }

    function plugin($name, $data){
    }

    /**
     * Write meta data to file.
     */
    function setmeta($id, $meta) {
        //dbglog("setmeta({$id}, ".print_r($meta,true).")");
        list($pageid, $boardid) = explode('#', $id);
        $fname = metaFN($pageid, ".mboard");
        io_makeFileDir($fname);
        io_lock($fname);
        if (@file_exists($fname))
            $metas = unserialize(file_get_contents($fname));
        else
            $metas = array();
        $fh = @fopen($fname, "wb");
        if (!$fh) {
            $this->message = "Can't write file '{$fname}'.";
            io_unlock($fname);
            return false;
        }
        $metas[$boardid] = $meta;
        fwrite($fh, serialize($metas));
        fclose($fh);
        io_unlock($fname);
        return true;
    }

    /**
     * Write session data.
     */
    function set_session($key, $val) {
        @session_start();
        $_SESSION['plugin_mboard'][$key] = $val;
        session_write_close();
    }

    /**
     * Read session data.
     */
    function get_session($key) {
        return $_SESSION['plugin_mboard'][$key];
    }

    /**
     * Read meta data from file.
     */
    function getmeta($id) {
        list($pageid, $boardid) = explode('#', $id);
        $fname = metaFN($pageid, ".mboard");
        if (!@file_exists($fname)) {
            $this->message = "Can't read file '{$fname}'.";
            return null;
        }
        $metas = unserialize(file_get_contents($fname));
        return $metas[$boardid];
    }

    /**
     * Format text to HTML.
     */
    function html_format($text) {
        return preg_replace('/\n/', '<br/>', hsc($text));
    }

    /**
     * Create message list by HTML format.
     */
    function html_list($id, $page) {
        $thread = preg_replace('/^.*#/', '', $id);
        $html = '<dl class="mboard" id="mboard_thread_'.hsc($thread).'">';
        $messages = $this->_list(array('id'=>$id, 'page'=>$page));
        if ($messages === FALSE) {
            return '<div class="error">'.
                $this->message[2].
                '</div>';
        }

        $uid = $this->_getuid();
        foreach ($messages as $m) {
            $menulink = $this->checkacl($id, $m) < AUTH_DELETE ?
                $m['MESSAGE_ID'] :
                '<a class="menu" href="javascript:void(0);" onclick="'.
                'javascript:plugin_mboard.remove(\''.$id.'\','.$m[MESSAGE_ID].','.$page.');'.
                '" title="'.hsc($this->getLang('removetitle')).'">'.
                $m['MESSAGE_ID'].
                '</a>';

            $html .= sprintf('<dt id="mboard_head_%s_%s">%s : '.
                             '<span class="user">%s</span> : %s</dt>',
                             $thread, $m['MESSAGE_ID'], $menulink,
                             hsc($m['MESSAGE_USER']), hsc($m['MESSAGE_DATE']));
            $html .= sprintf('<dd id="mboard_body_%s_%s">%s</dd>',
                             $thread, $m['MESSAGE_ID'],
                             $this->html_format($m['MESSAGE_TEXT']));
        }
        $html .= '</dl>';
        return $html;
    }

    /**
     * Create message list by XML format.
     */
    function xml_list($params=array()) {
        $doc = new DOMDocument('1.0', 'UTF-8');
        $result = $doc->appendChild($doc->createElement('result'));
        $code = $result->appendChild($doc->createElement('code', 0));
        $message = $result->appendChild($doc->createElement('message', 'OK'));
        $messages = $result->appendChild($doc->createElement('messages'));

        foreach ($this->_list($params) as $row) {
            $node = $messages->appendChild($doc->createElement('message'));
            $node->appendChild($doc->createElement('id', $row['MESSAGE_ID']));
            $node->appendChild($doc->createElement('user', $row['MESSAGE_USER']));
            list($date, $time) = explode(' ', $row['MESSAGE_DATE']);
            $node->appendChild($doc->createElement('date', $date));
            $node->appendChild($doc->createElement('time', $time));
            $node->appendChild($doc->createElement('text', hsc($row['MESSAGE_TEXT'])));
        }
        return $doc;
    }

    /**
     * Create message list by raw data.
     */
    function _list($params) {
        //dbglog("_list(".print_r($params,true).")");
        $id = $params['id'];
        $page = $params['page'];
        $meta = $this->getmeta($id);
        $size = $params['size'] ? $params['size'] : $meta['size'];
        $sort = $params['sort'] ? $params['sort'] : $meta['sort'];
        $prefix = $this->getConf('dbprefix');
        $sql = "SELECT * FROM {$prefix}MBOARD WHERE BOARD_ID=? AND STATUS=0";

        if ($sort == 'desc') {
            $sql .= " ORDER BY MESSAGE_ID DESC";
        } else {
            $sql .= " ORDER BY MESSAGE_ID ASC";
        }

        if (!$page) {
            $page = 1;
        } else if ($page < 0) { //last page
            $total = $this->_count($id);
            $pages = $total == 0 ? 1 : ceil($total / $size);
            $page = $pages + $page + 1;
        }

        if ($sort == 'desc') { //desc order
            $sql .= " LIMIT ".(($page - 1) * $size).",{$size}";
        } else { //asc order
            $total = $this->_count($id);
            $pages = $total == 0 ? 1 : ceil($total / $size);
            $size  = $size;
            $start = ($total - ($pages - $page + 1) * $size);
            if ($start < 0) {
                $size  = $size + $start;
                $start = 0;
            }
            $sql .= " LIMIT {$start},{$size}";
        }

        //dbglog($sql);
        $stmt = $this->_conn->prepare($sql);
        $stmt->bindParam(1, $id);
        $stmt->execute();
        if ($stmt->errorCode() != '00000') {
            $this->message = $stmt->errorInfo();
            return FALSE;
        }

        $result = array();
        while ($row = $stmt->fetch(PDO::FETCH_ASSOC)) {
            $result[] = $row;
        }

        return $result;
    }

    /**
     * Validate posted message.
     */
    function validate_message($id, $user, $mail, $text) {
        $errors = array();
        if ($this->getmeta($id) == null)
            $errors['id'][] = $this->getLang('error_id_not_exist');
        if (empty($user))
            $errors['user'][] = $this->getLang('error_user_empty');
        if (strlen($user) > 80)
            $errors['user'][] = $this->getLang('error_user_length');
        if (empty($text))
            $errors['text'][] = $this->getLang('error_text_empty');
        $textlength = $this->getConf('textlength');
        if ($textlength != 0 && strlen($text) > $textlength)
            $errors['text'][] = $this->getLang('error_text_length');
        $linelength = $this->getConf('linelength');
        if ($linelength != 0 && count(preg_split('/\n/',$text)) > $linelength)
            $errors['text'][] = $this->getLang('error_line_length');

        return $this->errors = $errors;
    }

    /**
     * Add new message.
     */
    function add_message($id, $user, $mail, $text) {
        //dbglog("add_message({$id}, {$user}, {$mail}, {$text})");
        $num = $this->_getnewnum($id);
        if ($num === FALSE) {
            return FALSE;
        }
        $errors = $this->validate_message($id, $user, $mail, $text);
        if (count($errors) > 0) {
            return FALSE;
        }

        $prefix = $this->getConf('dbprefix');
        $sql = "INSERT INTO {$prefix}MBOARD VALUES (?,?,?,?,?,now(),?,?,0)";
        $stmt = $this->_conn->prepare($sql);
        $stmt->bindParam(1, $id);
        $stmt->bindParam(2, $num);
        $stmt->bindParam(3, $user);
        $stmt->bindParam(4, $mail);
        $stmt->bindParam(5, $text);
        $stmt->bindParam(6, clientIP());
        $stmt->bindParam(7, $this->_getuid());
        $stmt->execute();
        if ($stmt->errorCode() != '00000') {
            $this->message = $stmt->errorInfo();
            return FALSE;
        }
        $this->log(1, sprintf($this->getLang('log_new_message'), $id, $num));

        //Refresh page modified time
        //This can't work good because DokuWiki says it's external_edit.
        //list($pageid, $boardid) = explode('#', $id);
        //touch(wikiFN($pageid));

        $sql = "SELECT * FROM {$prefix}MBOARD WHERE BOARD_ID=? AND MESSAGE_ID=?";
        $stmt = $this->_conn->prepare($sql);
        $stmt->bindParam(1, $id);
        $stmt->bindParam(2, $num);
        $stmt->execute();
        if ($stmt->errorCode() != '00000') {
            $this->message = $stmt->errorInfo();
            return FALSE;
        } else if ($row = $stmt->fetch(PDO::FETCH_ASSOC)) {
            return $row;
        }
    }

    function _getnewnum($id) {
        //dbglog("_getnewnum({$id})");
        $prefix = $this->getConf('dbprefix');
        $sql = "SELECT MAX(MESSAGE_ID)+1 MAX_ID FROM {$prefix}MBOARD WHERE BOARD_ID=?";
        $stmt = $this->_conn->prepare($sql);
        $stmt->bindParam(1, $id);
        $stmt->execute();
        if ($stmt->errorCode() != '00000') {
            $this->message = $stmt->errorInfo();
            return FALSE;
        } else if ($row = $stmt->fetch(PDO::FETCH_ASSOC)) {
            return $row['MAX_ID'] ? $row['MAX_ID'] : 1;
        } else {
            return 1;
        }
    }

    /**
     * Get user id.
     * If not login, automatically generate uid from session_id().
     * Such uid has '@' prefix.
     */
    function _getuid() {
        if ($_SERVER['REMOTE_USER']) {
            return $_SERVER['REMOTE_USER'];
        } else {
            $h = unpack("C4",pack('L', crc32(session_id())));
            return '@'.preg_replace('/=+$/', '',
                base64_encode(pack("C4", $h[0], $h[1], $h[2], $h[3])));
        }
    }

    /**
     * Get single message.
     */
    function get_message($id, $num) {
        $prefix = $this->getConf('dbprefix');
        $sql = "SELECT * FROM {$prefix}MBOARD WHERE BOARD_ID=? AND MESSAGE_ID=?";
        $stmt = $this->_conn->prepare($sql);
        $stmt->bindParam(1, $id);
        $stmt->bindParam(2, $num);
        $stmt->execute();
        if ($stmt->errorCode() != '00000') {
            $this->message = $stmt->errorInfo();
            return FALSE;
        }
        $row = $stmt->fetch(PDO::FETCH_ASSOC);
        return $row;
    }

    /**
     * Remove message.
     */
    function remove_message($id, $num) {
        //dbglog("remove({$id})");
        $msg = $this->get_message($id, $num);
        if ($this->checkacl($id, $msg) < AUTH_DELETE) {
            $this->message = "access denied.";
            return false;
        }

        $prefix = $this->getConf('dbprefix');
        $sql = "UPDATE {$prefix}MBOARD SET STATUS=1 WHERE BOARD_ID=? AND MESSAGE_ID=?";
        $stmt = $this->_conn->prepare($sql);
        $stmt->bindParam(1, $id);
        $stmt->bindParam(2, $num);
        $stmt->execute();
        if ($stmt->errorCode() != '00000') {
            $this->message = $stmt->errorInfo();
            return false;
        }
        $this->log(1, sprintf($this->getLang('log_remove_message'), $id, $num));
        return $stmt->rowCount();
    }

    /**
     * Create index by HTML format.
     */
    function html_index($id, $page) {
        $meta = $this->getmeta($id);
        $idx = $this->_index($id, $page);
        if (count($idx['index']) <= 1)
            return '';

        $html = '<div class="mboard-index">';
        $html .= $idx['prev'] ?
            $this->_anchor('mboard-prev', $id, $idx['prev'], '&lt;&lt;') :
            '';
        foreach ($idx['index'] as $i) {
            if ($i['curr']) {
                $html .= '<span class="mboard-curr">'.$i['page'].'</span>';
            } else {
                $html .= $this->_anchor('mboard-index', $id, $i['page'], $i['page']);
            }
        }
        $html .= $idx['next'] ?
            $this->_anchor('mboard-next', $id, $idx['next'], '&gt;&gt;') :
            '';
        $html .= '</div>';

        return $html;
    }

    function _anchor($class, $id, $page, $text) {
        return '<a class="'.$class.'" href="javascript:void(0)" onclick="plugin_mboard.list(\''.$id.'\','.$page.')">'.$text.'</a>';
    }

    /**
     * Create index by raw data.
     */
    function _index($id, $page) {
        //dbglog("_index({$id},{$page})");
        $meta = $this->getmeta($id);
        $total = $this->_count($id);
        $pages = $total == 0 ? 1 : ceil($total / $meta['size']);
        if ($page < 0) {
            $page = $pages + $page + 1;
        }
        $result = array('prev' => (($page - 1) > 0 ? $page - 1 : 0),
                        'next' => (($page + 1) <= $pages ? $page + 1 : 0));
        $result['index'] = array();
        for ($i = 1; $i <= $pages; $i++) {
            $result['index'][] = array('page' => $i,
                                       'curr' => ($i == $page ? 1 : 0));
        }
        return $result;
    }

    /**
     * Get total messages count.
     */
    function _count($id) {
        //dbglog("_count({$id})");
        $prefix = $this->getConf('dbprefix');
        $sql = "SELECT COUNT(*) TOTAL FROM {$prefix}MBOARD WHERE BOARD_ID=? AND STATUS=0";
        $stmt = $this->_conn->prepare($sql);
        $stmt->bindParam(1, $id);
        $stmt->execute();
        if ($stmt->errorCode() != '00000') {
            $this->message = $stmt->errorInfo();
            return FALSE;
        }
        $row = $stmt->fetch(PDO::FETCH_ASSOC);
        return $row['TOTAL'];
    }

    /**
     * Create edit form by HTML format.
     */
    function html_form($id, $form=array()) {
        if ($this->checkacl($id) < AUTH_EDIT)
            return '';

        $meta = $this->getmeta($id);
        $page = $meta['sort'] == 'desc' ? 1 : -1;
        $token = md5(session_id());

        return <<<EOF
<form action="javascript:void(0);">
<input type="hidden" name="call" value="mboard"/>
<input type="hidden" name="cmd" value="add"/>
<input type="hidden" name="id" value="{$id}"/>
<input type="hidden" name="page" value="{$page}"/>
<input type="hidden" name="token" value="{$token}"/>
<table class="mboard-form">
<tr><th>お名前</th>
    <td><input type="text" class="mboard-edit" name="user" value="{$form['user']}" style="width:150px;"/></td></tr>
<tr><th>メッセージ</th>
    <td><textarea name="text" class="mboard-edit" style="width:90%;height:100px;">{$form['text']}</textarea></td></tr>
<tr><td></td>
    <td><input type="button" class="mboard-edit" value="送信" onclick="plugin_mboard.submit(this)" style="width:100px;"/></td></tr>
</table>
</form>
EOF;
    }

    /**
     * Returns access level to board or message.
     */
    function checkacl($boardid, $msg=FALSE) {
        //dbglog("checkacl({$boardid},".print_r($msg,true).")");
        $meta = $this->getmeta($boardid);
        $pageid = preg_replace('/#.*$/', '', $boardid);

        if ($msg === FALSE) { //for board
            switch ($meta['post']) {
            case 'public':
                return AUTH_EDIT;
            case 'user':
                return $_SERVER['REMOTE_USER'] ? AUTH_EDIT : AUTH_READ;
            default:
                return auth_quickaclcheck($pageid);
            }
        } else { //for message
            if (ctype_digit($msg)) { //id specified.
                $msg = $this->get_message($boardid, $msg);
            }
            switch ($meta['post']) {
            case 'public':
                return $msg['USER_ID'] == $this->_getuid() ?
                    AUTH_DELETE :
                    auth_quickaclcheck($pageid);
            default:
                return auth_quickaclcheck($pageid);
            }
        }
    }

    function log($level, $message) {
        $data = array('[mboard]', $level, $message);
        trigger_event('LOGGER_LOG_WRITE', $data);
    }
}
