<?php
/*
 * Mboard Syntax Plugin.
 *
 * @license  Apache 2 (http://www.apache.org/licenses/LICENSE-2.0)
 * @author   Kazuyuki Matsuda / kazm.atmark.shorindo.com
 */

if (!defined('DOKU_INC')) die();
if (!defined('DOKU_LF')) define('DOKU_LF', "\n");
if (!defined('DOKU_TAB')) define('DOKU_TAB', "\t");
if (!defined('DOKU_PLUGIN')) define('DOKU_PLUGIN',DOKU_INC.'lib/plugins/');
require_once(DOKU_PLUGIN.'syntax.php');

class syntax_plugin_mboard extends DokuWiki_Syntax_Plugin {

	function getInfo() {
        return confToHash(dirname(__FILE__).'/plugin.info.txt');
	}

	function getType() { return 'container'; }

	function getSort() { return 300; }

	function getPType() { return 'block';}

	function connectTo($mode) {
		$this->Lexer->addSpecialPattern('\{\{mboard>.*?\}\}', $mode, 'plugin_mboard');
	}

	function handle($match, $state, $pos, &$handler) {
		$params = array();
		if (preg_match("/^{{mboard>(.*)}}$/", $match, $m)) {
			foreach (explode('&', $m[1]) as $p) {
                list($key, $val) = preg_split('/=/', $p, 2);
				$params[$key] = $val;
			}
		}

		return array($state, $params);
    }

	function render($mode, &$renderer, $data) {
		if($mode == 'xhtml'){
			list($state, $params) = $data;
            $renderer->nocache();
            if (!$params['id']) {
                $renderer->doc .=
                    '<div class="error">'.
                    $this->getLang('noid').
                    '</div>';
            }
            if (!$params['size']) {
                $params['size'] = $this->getConf('pagesize');
            }
            if (!$params['sort']) {
                $params['sort'] = 'asc';
            }
            if ($params['sort'] == 'asc') {
                $page = -1;
            } else {
                $page = 1;
            }
            try {
                $helper = plugin_load('helper', 'mboard');
                if (!$helper)
                    return false;
                if (strpos($params['id'], '#') === FALSE)
                    $params['id'] = getID().'#'.$params['id'];
                $sid = preg_replace(array('/^.*#/', '/[^a-zA-Z]/'),
                                    array('', '_'),
                                    $params['id']);
                $helper->setmeta($params['id'], array('size' => $params['size'],
                                                      'sort' => $params['sort'],
                                                      'post' => $params['post']));
                $form = array('user' => $helper->get_session('user'));
                if (!$form['user'] && isset($_SERVER['REMOTE_USER'])) {
                    global $INFO;
                    $form['user'] = $INFO['userinfo']['name'];
                }
                
                if ($params['sort'] == 'desc')
                    $renderer->doc .=
                        '<div id="mboard__'.$sid.'" class="mboard">'.
                        $helper->html_form($params['id'], $form).
                        $helper->html_list($params['id'], $page).
                        $helper->html_index($params['id'], $page).
                        '</div>';
                else
                    $renderer->doc .=
                        '<div id="mboard__'.$sid.'" class="mboard">'.
                        $helper->html_list($params['id'], $page).
                        $helper->html_index($params['id'], $page).
                        $helper->html_form($params['id'], $form).
                        '</div>';
            } catch (Exception $ex) {
                //TODO if debug mode
                $renderer->doc .= '<div class="error">'.$ex->getMessage().'</div>';
            }
        }
    }
}
